/*----------------------------------------------------------------------
  File    : ccnpsp.c
  Contents: generate or estimate a pattern spectrum (CoCoNAD)
  Author  : Christian Borgelt
  History : 2015.08.13 file created
----------------------------------------------------------------------*/
#ifndef __CCNPSP__
#define __CCNPSP__
#include "coconad.h"
#include "patspec.h"

/*----------------------------------------------------------------------
  Preprocessor Definitions
----------------------------------------------------------------------*/
/* --- surrogate data methods --- */
#define CCN_IDENTITY    0       /* identity (keep original data) */
#define CCN_RANDOM      1       /* spike time randomization */
#define CCN_DITHER      2       /* point/spike time dithering */
#define CCN_SHIFT       3       /* train shifting/dithering */
#define CCN_KERNEST     4       /* sampling from kernel estimate */
#define CCN_PERMUTE     5       /* dithered spike permutation */
#define CCN_BLOCK       6       /* dithered blocked permutations */

/*----------------------------------------------------------------------
  Type Definitions
----------------------------------------------------------------------*/
typedef void PRGREPFN (long cnt, void *data);
                                /* progress reporting function */

/*----------------------------------------------------------------------
  Functions
----------------------------------------------------------------------*/
extern PATSPEC* ccn_genpsp (TRAINSET *trnset, int target, double width,
                            double supp,
#ifdef CCN_OVERLAP              /* if influence map overlap support */
                            double mix,
#endif                          /* additional mixing parameter */
                            ITEM zmin, ITEM zmax,
                            int algo, int mode, size_t cnt,
                            double beg, double end, int surr, int rand,
                            double sigma, double delta, long seed,
                            int cpus, PRGREPFN *rep, void* data);

extern PATSPEC* ccn_estpsp (TRAINSET *trnset, int target, double width,
                            double supp, ITEM zmin, ITEM zmax,
                            size_t equiv, double alpha, size_t smpls,
                            long seed);
#endif
