#!/usr/bin/python

from sys        import argv, stderr
from os         import remove, devnull
from subprocess import call, check_output
from time       import time
from psts       import genpsts
from ccnovl     import coconad, cocojim

#-----------------------------------------------------------------------

def run_pipe (prog, n, null):
    res = []
    for line in check_output(prog +['-'],stderr=null).split('\n')[:-1]:
        line = line.split()
        res.append(tuple([[int(x)  for x in line[:-n]]]
                       + [float(x) for x in line[-n:]]))
    return res

#-----------------------------------------------------------------------

def run_file (prog, n, null):
    call(prog +['tmp.txt'], stderr=null)
    res = []
    with open('tmp.txt', 'rt') as inp:
        for line in inp:
            line = line.split()
            res.append(tuple([[int(x)  for x in line[:-n]]]
                           + [float(x) for x in line[-n:]]))
    remove('tmp.txt')
    return res

#-----------------------------------------------------------------------

if __name__ == '__main__':
    runs = int(argv[1]) if len(argv) > 1 else 1
    ok   = True

    stderr.write('generating data ... '); t = time()
    psts = zip(range(1,101), genpsts(100, 3, 20))
    with open('data.txt', 'w') as out:
        for n,x in psts:
            for s in x: out.write('%d %g\n' % (n,s))
    psts = dict()
    with open('data.txt', 'r') as inp:
        for line in inp:
            n,s = line.split()
            n,s = int(n),float(s)
            if n in psts: psts[n].append(s)
            else:         psts[n] = [s]
    psts = psts.items()
    # The generated data needs to be re-read to compensate
    # rounding errors resulting from the writing.

    stderr.write('repetitions: %d\n' % runs)

    stderr.write('ccn-ovl sim   ... '); t = time()
    for r in range(runs):
        iipats = cocojim(psts, width=0.003, supp=1, simcov='j',
                         sim=0.01, zmin=2, algo='i', report='aw')
    stderr.write('done [%d, %.3fs].\n' % (len(iipats),time()-t))
    iipats = set([(tuple(sorted(list(s))),
                   float('%g' % x),float('%g' % y))
                  for s,x,y in iipats])

    prg = ['ccn-ovl', '-w0.003', '-s1', '-m2',
           '-ij', '-c0.01', '-v %a %w', 'data.txt']
    stderr.write('file sim      ... '); t = time()
    with open(devnull, 'w') as null:
        for r in range(runs): expats = run_file(prg, 2, null)
    stderr.write('done [%d, %.3fs].\n' % (len(expats),time()-t))

    stderr.write('pipe sim      ... '); t = time()
    with open(devnull, 'w') as null:
        for r in range(runs): expats = run_pipe(prg, 2, null)
    stderr.write('done [%d, %.3fs].\n' % (len(expats),time()-t))
    expats = set([(tuple(sorted(list(s))),
                   float('%g' % x),float('%g' % y))
                  for s,x,y in expats])

    if iipats == expats: print('ok')
    else: ok = False;    print('fail')

    stderr.write('ccn-ovl sim e ... '); t = time()
    for r in range(runs):
        iipats = cocojim(psts, width=0.003, supp=1, simcov='j',
                         sim=0.01, zmin=2, algo='i', mode='e',
                         report='aw')
    stderr.write('done [%d, %.3fs].\n' % (len(iipats),time()-t))
    iipats = set([(tuple(sorted(list(s))),
                   float('%g' % x),float('%g' % y))
                  for s,x,y in iipats])

    prg = ['ccn-ovl', '-w0.003', '-s1', '-m2',
           '-ij', '-c0.01', '-e', '-v %a %w', 'data.txt']
    stderr.write('file sim e    ... '); t = time()
    with open(devnull, 'w') as null:
        for r in range(runs): expats = run_file(prg, 2, null)
    stderr.write('done [%d, %.3fs].\n' % (len(expats),time()-t))

    stderr.write('pipe sim e    ... '); t = time()
    with open(devnull, 'w') as null:
        for r in range(runs): expats = run_pipe(prg, 2, null)
    stderr.write('done [%d, %.3fs].\n' % (len(expats),time()-t))
    expats = set([(tuple(sorted(list(s))),
                   float('%g' % x),float('%g' % y))
                  for s,x,y in expats])

    if iipats == expats: print('ok')
    else: ok = False;    print('fail')

    if ok: remove('data.txt')
